@@ -288,7 +288,12 @@ bool SourceData::isText()
 {
    return m_normalData.m_bIsText;
 }
- 
+
+bool SourceData::isIncompleteConversion()
+{
+   return m_normalData.m_bIncompleteConversion;
+}
+
 bool SourceData::isFromBuffer()
 {
    return !m_fileAccess.isValid();
@@ -310,6 +315,7 @@ void SourceData::FileData::reset()
    m_size = 0;
    m_vSize = 0;
    m_bIsText = true;
+   m_bIncompleteConversion = false;
    m_eLineEndStyle = eLineEndStyleUndefined;
 }
 
@@ -380,24 +386,90 @@ static bool convertFileEncoding( const QString& fileNameIn, QTextCodec* pCodecIn
    return true;
 }
 
+static QTextCodec* getEncodingFromTag( const QByteArray& s, const QByteArray& encodingTag )
+{   
+   int encodingPos = s.indexOf( encodingTag );
+   if ( encodingPos>=0 )
+   {
+      int apostrophPos = s.indexOf( '"', encodingPos + encodingTag.length() );
+      int apostroph2Pos = s.indexOf( '\'', encodingPos + encodingTag.length() );
+      char apostroph = '"';
+      if ( apostroph2Pos>=0 && ( apostrophPos<0 || (apostrophPos>=0 && apostroph2Pos < apostrophPos) ) )
+      {
+         apostroph = '\'';
+         apostrophPos = apostroph2Pos;
+      }
+
+      int encodingEnd = s.indexOf( apostroph, apostrophPos+1 );
+      if ( encodingEnd>=0 ) // e.g.: <meta charset="utf-8"> or <?xml version="1.0" encoding="ISO-8859-1"?>
+      {
+         QByteArray encoding = s.mid( apostrophPos+1, encodingEnd - (apostrophPos + 1) );
+         return QTextCodec::codecForName( encoding );
+      }
+      else // e.g.: <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
+      {
+         QByteArray encoding = s.mid( encodingPos+encodingTag.length(), apostrophPos - ( encodingPos+encodingTag.length() ) );
+         return QTextCodec::codecForName( encoding );
+      }
+   }
+   return 0;
+}
+
 static QTextCodec* detectEncoding( const char* buf, qint64 size, qint64& skipBytes )
 {
    if (size>=2)
    {
-      skipBytes = 2;
       if (buf[0]=='\xFF' && buf[1]=='\xFE' )
+      {
+         skipBytes = 2;
          return QTextCodec::codecForName( "UTF-16LE" );
+      }
 
       if (buf[0]=='\xFE' && buf[1]=='\xFF' )
+      {
+         skipBytes = 2;
          return QTextCodec::codecForName( "UTF-16BE" );
+      }
    }
    if (size>=3)
    {
-      skipBytes = 3;
       if (buf[0]=='\xEF' && buf[1]=='\xBB' && buf[2]=='\xBF' )
+      {
+         skipBytes = 3;
          return QTextCodec::codecForName( "UTF-8-BOM" );
+      }
    }
    skipBytes = 0;
+   QByteArray s( buf, size );
+   int xmlHeaderPos = s.indexOf( "<?xml" );
+   if ( xmlHeaderPos >= 0 )
+   {
+      int xmlHeaderEnd = s.indexOf( "?>", xmlHeaderPos );
+      if ( xmlHeaderEnd>=0 )
+      {
+         QTextCodec* pCodec = getEncodingFromTag( s.mid( xmlHeaderPos, xmlHeaderEnd - xmlHeaderPos ), "encoding=" );
+         if (pCodec)
+            return pCodec;
+      }
+   }
+   else // HTML
+   {
+      int metaHeaderPos = s.indexOf( "<meta" );
+      while ( metaHeaderPos >= 0)
+      {
+         int metaHeaderEnd = s.indexOf( ">", metaHeaderPos );
+         if (metaHeaderEnd>=0)
+         {
+            QTextCodec* pCodec = getEncodingFromTag( s.mid( metaHeaderPos, metaHeaderEnd - metaHeaderPos ), "charset=" );
+            if (pCodec)
+               return pCodec;
+
+            metaHeaderPos = s.indexOf( "<meta", metaHeaderEnd );
+         }
+         else
+            break;
+      }
+   }
    return 0;
 }
 
@@ -406,7 +478,7 @@ QTextCodec* SourceData::detectEncoding( const QString& fileName, QTextCodec* pFa
    QFile f(fileName);
    if ( f.open(QIODevice::ReadOnly) )
    {
-      char buf[4];
+      char buf[200];
       qint64 size = f.read( buf, sizeof(buf) );
       qint64 skipBytes = 0;
       QTextCodec* pCodec = ::detectEncoding( buf, size, skipBytes );
@@ -416,6 +488,111 @@ QTextCodec* SourceData::detectEncoding( const QString& fileName, QTextCodec* pFa
    return pFallbackCodec;
 }
 
+/* Split the command line into arguments.
+ * Normally split at white space separators except when quoting with " or '.
+ * Backslash is treated as meta character within single quotes ' only.
+ * Detect parsing errors like unclosed quotes.
+ * The first item in the list will be the command itself.
+ * Returns the error reasor as string or an empty string on success.
+ * Eg. >"1" "2"<           => >1<, >2<
+ * Eg. >'\'\\'<            => >'\<   backslash is a meta character between single quotes
+ * Eg. > "\\" <            => >\\<   but not between double quotes
+ * Eg. >"c:\sed" 's/a/\' /g'<  => >c:\sed<, >s/a/' /g<
+ */
+static QString getArguments( QString cmd, QString& program, QStringList& args )
+{
+   program = QString();
+   args.clear();
+   for ( int i=0; i<cmd.length(); ++i )
+   {
+      while ( i<cmd.length() && cmd[i].isSpace() )
+      {
+         ++i;
+      }
+      if ( cmd[i]=='"' || cmd[i]=='\'' ) // argument beginning with a quote
+      {
+         QChar quoteChar = cmd[i];
+         ++i;
+         int argStart = i;
+         bool bSkip = false;
+         while ( i<cmd.length() && ( cmd[i]!=quoteChar || bSkip ) )
+         {
+            if ( bSkip )
+            {
+               bSkip = false;
+               if ( cmd[i]=='\\' || cmd[i]==quoteChar )
+               {
+                  cmd.remove( i-1, 1 );  // remove the backslash '\'
+                  continue;
+               }
+            }
+            else if ( cmd[i]=='\\' && quoteChar=='\'')
+               bSkip = true;
+            ++i;
+         }
+         if ( i<cmd.length() )
+         {
+            args << cmd.mid( argStart, i-argStart );
+            if ( i+1<cmd.length() && !cmd[i+1].isSpace() )
+               return i18n("Expecting space after closing apostroph.");
+         }
+         else
+            return i18n("Not matching apostrophs.");
+         continue;
+      }
+      else
+      {            
+         int argStart = i;
+         //bool bSkip = false;
+         while ( i<cmd.length() && ( !cmd[i].isSpace() /*|| bSkip*/ ) )
+         {
+            /*if ( bSkip )
+            {
+               bSkip = false;
+               if ( cmd[i]=='\\' || cmd[i]=='"' || cmd[i]=='\'' || cmd[i].isSpace() )
+               {
+                  cmd.remove( i-1, 1 );  // remove the backslash '\'
+                  continue;
+               }
+            }
+            else if ( cmd[i]=='\\' )
+               bSkip = true;
+            else */
+            if ( cmd[i]=='"' || cmd[i]=='\'' )
+               return i18n("Unexpected apostroph within argument.");
+            ++i;
+         }
+         args << cmd.mid( argStart, i-argStart );
+      }      
+   }
+   if ( args.isEmpty() )
+      return i18n("No program specified.");
+   else
+   {
+      program = args[0];
+      args.pop_front();
+      #ifdef WIN32
+      if ( program=="sed" )
+      {
+         QString prg = QCoreApplication::applicationDirPath() + "/bin/sed.exe"; // in subdir bin
+         if ( QFile::exists( prg ) )
+         {
+            program = prg;
+         }
+         else
+         {
+            prg = QCoreApplication::applicationDirPath() + "/sed.exe"; // in same dir
+            if ( QFile::exists( prg ) )
+            {
+               program = prg;
+            }
+         }
+      }
+      #endif 
+   }
+   return QString();
+}
+
 void SourceData::readAndPreprocess( QTextCodec* pEncoding, bool bAutoDetectUnicode )
 {
    m_pEncoding = pEncoding;
@@ -462,7 +639,7 @@ void SourceData::readAndPreprocess( QTextCodec* pEncoding, bool bAutoDetectUnico
    if ( faIn.exists() ) // fileInSize > 0 )
    {
 
-#ifdef _WIN32
+#if defined(_WIN32) || defined(Q_OS_OS2)
       QString catCmd = "type";
       fileNameIn1.replace( '/', "\\" );
 #else
@@ -493,17 +670,25 @@ void SourceData::readAndPreprocess( QTextCodec* pEncoding, bool bAutoDetectUnico
          QProcess ppProcess;
          ppProcess.setStandardInputFile( fileNameInPP );
          ppProcess.setStandardOutputFile( fileNameOut1 );
-         ppProcess.start( ppCmd );
-         ppProcess.waitForFinished(-1);
+         QString program;
+         QStringList args;
+         QString errorReason = getArguments(ppCmd, program, args);
+         if ( errorReason.isEmpty() )
+         {
+            ppProcess.start( program, args );
+            ppProcess.waitForFinished(-1);
+         }
+         else
+            errorReason = "\n("+errorReason+")";
          //QString cmd = catCmd + " \"" + fileNameInPP + "\" | " + ppCmd  + " >\"" + fileNameOut1+"\"";
          //::system( encodeString(cmd) );
-         bool bSuccess = m_normalData.readFile( fileNameOut1 );
+         bool bSuccess = errorReason.isEmpty() && m_normalData.readFile( fileNameOut1 );
          if ( fileInSize >0 && ( !bSuccess || m_normalData.m_size==0 ) )
          {
             KMessageBox::error(m_pOptionDialog, 
                i18n("Preprocessing possibly failed. Check this command:\n\n  %1"
                   "\n\nThe preprocessing command will be disabled now."
-               ).arg(ppCmd) );
+               ).arg(ppCmd) + errorReason );
             m_pOptionDialog->m_PreProcessorCmd = "";
             m_normalData.readFile( fileNameIn1 );
             pEncoding1 = m_pEncoding;
@@ -533,17 +718,25 @@ void SourceData::readAndPreprocess( QTextCodec* pEncoding, bool bAutoDetectUnico
          QProcess ppProcess;
          ppProcess.setStandardInputFile( fileNameInPP );
          ppProcess.setStandardOutputFile( fileNameOut2 );
-         ppProcess.start( ppCmd );
-         ppProcess.waitForFinished(-1);
+         QString program;
+         QStringList args;
+         QString errorReason = getArguments(ppCmd, program, args);
+         if ( errorReason.isEmpty() )
+         {
+            ppProcess.start( program, args );
+            ppProcess.waitForFinished(-1);
+         }
+         else
+            errorReason = "\n("+errorReason+")";
          //QString cmd = catCmd + " \"" + fileNameInPP + "\" | " + ppCmd  + " >\"" + fileNameOut2 + "\"";
          //::system( encodeString(cmd) );
-         bool bSuccess = m_lmppData.readFile( fileNameOut2 );
+         bool bSuccess = errorReason.isEmpty() && m_lmppData.readFile( fileNameOut2 );
          if ( FileAccess(fileNameIn2).size()>0 && ( !bSuccess || m_lmppData.m_size==0 ) )
          {
             KMessageBox::error(m_pOptionDialog, 
                i18n("The line-matching-preprocessing possibly failed. Check this command:\n\n  %1"
                     "\n\nThe line-matching-preprocessing command will be disabled now."
-                   ).arg(ppCmd) );
+                   ).arg(ppCmd) + errorReason );
             m_pOptionDialog->m_LineMatchingPreProcessorCmd = "";
             m_lmppData.readFile( fileNameIn2 );
          }
@@ -629,8 +822,8 @@ void SourceData::FileData::preprocess( bool bPreserveCR, QTextCodec* pEncoding )
    {
       if ( m_pBuf[i]=='\n' )
       {
-         if ( i>0 && m_pBuf[i-1]=='\r' ||  // normal file
-              i>3 && m_pBuf[i-1]=='\0' && m_pBuf[i-2]=='\r' && m_pBuf[i-3]=='\0') // 16-bit unicode
+         if ( (i>0 && m_pBuf[i-1]=='\r') ||  // normal file
+              (i>3 && m_pBuf[i-1]=='\0' && m_pBuf[i-2]=='\r' && m_pBuf[i-3]=='\0')) // 16-bit unicode: TODO only little endian covered here
             m_eLineEndStyle = eLineEndStyleDos;
          else
             m_eLineEndStyle = eLineEndStyleUnix;
@@ -643,6 +836,14 @@ void SourceData::FileData::preprocess( bool bPreserveCR, QTextCodec* pEncoding )
       skipBytes=0;
 
    QByteArray ba = QByteArray::fromRawData( m_pBuf+skipBytes, m_size-skipBytes );
+   if ( m_eLineEndStyle == eLineEndStyleUndefined ) // normally only for one liners except when old mac line end style is used
+   {
+      for( int j=0; j<ba.size(); ++j ) // int because QByteArray does not support operator[](qint64)
+      {
+         if ( ba[j]=='\r' )
+            ba[j]='\n'; // We only fix the old mac line end style, but leave it as "undefined"
+      }
+   }
    QTextStream ts( ba, QIODevice::ReadOnly );
    ts.setCodec( pEncoding);
    ts.setAutoDetectUnicode( false );
@@ -654,6 +855,7 @@ void SourceData::FileData::preprocess( bool bPreserveCR, QTextCodec* pEncoding )
 
    m_bIsText = true;
    int lines = 1;
+   m_bIncompleteConversion = false;
    for( i=0; i<ucSize; ++i )
    {
       if ( isLineOrBufEnd(p,i,ucSize) )
@@ -664,6 +866,10 @@ void SourceData::FileData::preprocess( bool bPreserveCR, QTextCodec* pEncoding )
       {
          m_bIsText = false;
       }
+      if ( p[i]==QChar::ReplacementCharacter )
+      {
+         m_bIncompleteConversion = true;
+      }
    }
 
    m_v.resize( lines+5 );
@@ -1200,13 +1406,13 @@ static bool isValidMove( ManualDiffHelpList* pManualDiffHelpList, int line1, int
 
          if ( l1>=0 && l2>=0 )
          {
-            if ( line1>=l1 && line2<l2 || line1<l1 && line2>=l2 )
+            if ( (line1>=l1 && line2<l2) || (line1<l1 && line2>=l2) )
                return false;
             l1 = winIdx1 == 1 ? mdhe.lineA2 : winIdx1==2 ? mdhe.lineB2 : mdhe.lineC2 ;
             l2 = winIdx2 == 1 ? mdhe.lineA2 : winIdx2==2 ? mdhe.lineB2 : mdhe.lineC2 ;
             ++l1;
             ++l2;
-            if ( line1>=l1 && line2<l2 || line1<l1 && line2>=l2 )
+            if ( (line1>=l1 && line2<l2) || (line1<l1 && line2>=l2) )
                return false;
          }
       }
@@ -1405,9 +1611,9 @@ void calcDiff3LineListTrim(
    {
       if ( iMDHL!=pManualDiffHelpList->end() )
       {
-         if ( i3->lineA >= 0 && i3->lineA==iMDHL->lineA1 || 
-              i3->lineB >= 0 && i3->lineB==iMDHL->lineB1 || 
-              i3->lineC >= 0 && i3->lineC==iMDHL->lineC1 )
+         if ( (i3->lineA >= 0 && i3->lineA==iMDHL->lineA1) || 
+              (i3->lineB >= 0 && i3->lineB==iMDHL->lineB1) || 
+              (i3->lineC >= 0 && i3->lineC==iMDHL->lineC1) )
          {
             i3A = i3;
             i3B = i3;
@@ -1660,7 +1866,7 @@ void debugLineCheck( Diff3LineList& d3ll, int size, int idx )
          {
             KMessageBox::error(0, i18n(
                "Data loss error:\n"
-               "If it is reproducable please contact the author.\n"
+               "If it is reproducible please contact the author.\n"
                ), i18n("Severe Internal Error") );
             assert(false);
             std::cerr << "Severe Internal Error.\n";
@@ -1674,7 +1880,7 @@ void debugLineCheck( Diff3LineList& d3ll, int size, int idx )
    {
       KMessageBox::error(0, i18n(
          "Data loss error:\n"
-         "If it is reproducable please contact the author.\n"
+         "If it is reproducible please contact the author.\n"
          ), i18n("Severe Internal Error") );
       assert(false);
       std::cerr << "Severe Internal Error.\n";
@@ -1880,7 +2086,7 @@ void fineDiff(
       else if (selector==2){ k1=i->lineB; k2=i->lineC; }
       else if (selector==3){ k1=i->lineC; k2=i->lineA; }
       else assert(false);
-      if( k1==-1 && k2!=-1  ||  k1!=-1 && k2==-1 ) bTextsTotalEqual=false;
+      if( (k1==-1 && k2!=-1)  ||  (k1!=-1 && k2==-1) ) bTextsTotalEqual=false;
       if( k1!=-1 && k2!=-1 )
       {
          if ( v1[k1].size != v2[k2].size || memcmp( v1[k1].pLine, v2[k2].pLine, v1[k1].size<<1)!=0 )
