@@ -68,9 +68,17 @@ public:
       m_scrollDeltaY = 0;
       m_bMyUpdate = false;
       m_bSelectionInProgress = false;
+      m_pTextCodec = 0;
+      #if defined(_WIN32) || defined(Q_OS_OS2)
+      m_eLineEndStyle = eLineEndStyleDos;
+      #else
+      m_eLineEndStyle = eLineEndStyleUnix;
+      #endif
    }
    DiffTextWindow* m_pDiffTextWindow;
    DiffTextWindowFrame* m_pDiffTextWindowFrame;
+   QTextCodec* m_pTextCodec;
+   e_LineEndStyle m_eLineEndStyle;
 
    bool m_bPaintingAllowed;
    const LineData* m_pLineData;
@@ -145,7 +153,7 @@ DiffTextWindow::DiffTextWindow(
 {
    setObjectName(QString("DiffTextWindow%1").arg(winIdx));
    setAttribute( Qt::WA_OpaquePaintEvent );
-   setAttribute( Qt::WA_PaintOnScreen );
+   //setAttribute( Qt::WA_PaintOnScreen );
 
    d = new DiffTextWindowData(this);
    d->m_pDiffTextWindowFrame = pParent;
@@ -153,7 +161,7 @@ DiffTextWindow::DiffTextWindow(
    setAcceptDrops( true );
 
    d->m_pOptionDialog = pOptionDialog;
-   init( 0, 0, 0, 0, 0, false );
+   init( 0, 0, d->m_eLineEndStyle, 0, 0, 0, 0, false );
 
    setMinimumSize(QSize(20,20));
 
@@ -172,6 +180,8 @@ DiffTextWindow::~DiffTextWindow()
 
 void DiffTextWindow::init(
    const QString& filename,
+   QTextCodec* pTextCodec,
+   e_LineEndStyle eLineEndStyle,
    const LineData* pLineData,
    int size,
    const Diff3LineVector* pDiff3LineVector,
@@ -202,6 +212,9 @@ void DiffTextWindow::init(
    d->m_selection.oldLastLine = -1;
    d->m_selection.lastLine = -1;
 
+   d->m_pTextCodec = pTextCodec;
+   d->m_eLineEndStyle = eLineEndStyle;
+
    update();
    d->m_pDiffTextWindowFrame->init();
 }
@@ -378,7 +391,7 @@ void DiffTextWindow::setFastSelectorRange( int line1, int nofLines )
 void DiffTextWindow::showStatusLine(int line )
 {
    int d3lIdx = convertLineToDiff3LineIdx( line );
-   if(d3lIdx >= 0 && d3lIdx<(int)d->m_pDiff3LineVector->size() )
+   if( d->m_pDiff3LineVector!=0 && d3lIdx >= 0 && d3lIdx<(int)d->m_pDiff3LineVector->size() )
    {
       const Diff3Line* pD3l = (*d->m_pDiff3LineVector)[d3lIdx];
       if ( pD3l != 0 )
@@ -552,12 +565,13 @@ void DiffTextWindow::mouseMoveEvent ( QMouseEvent * e )
       }
       if ( e->y() < 0 )          deltaY = -1 - sqr( e->y() ) / sqr(fm.height());
       if ( e->y() > height() )   deltaY = +1 + sqr( e->y() - height() ) / sqr(fm.height());
-      if ( deltaX != 0 && d->m_scrollDeltaX!=deltaX || deltaY!= 0 && d->m_scrollDeltaY!=deltaY )
+      if ( (deltaX != 0 && d->m_scrollDeltaX!=deltaX) || (deltaY!= 0 && d->m_scrollDeltaY!=deltaY) )
       {
          d->m_scrollDeltaX = deltaX;
          d->m_scrollDeltaY = deltaY;
          emit scroll( deltaX, deltaY );
-         killTimer( d->m_delayedDrawTimer );
+         if (d->m_delayedDrawTimer)
+            killTimer( d->m_delayedDrawTimer );
          d->m_delayedDrawTimer = startTimer(50);
       }
       else
@@ -711,6 +725,66 @@ bool Selection::lineWithin( int l )
    return ( l1 <= l && l <= l2 );
 }
 
+class DrawTextOptimizer
+// Drawing continuous text is much faster than drawing one char after another.
+{
+private:
+   QFont m_font;
+   QPen m_pen;
+   QString m_text;
+   QPoint m_pos;
+   MyPainter* m_pPainter;
+   QFontMetrics m_fm;
+   void draw()
+   {
+      if ( m_pPainter && !m_text.isEmpty() )
+      {
+         m_pPainter->setFont( m_font );
+         m_pPainter->setPen( m_pen );
+         m_pPainter->drawText( m_pos.x(), m_pos.y(), m_text, true );
+         m_text = QString();
+      }
+   }
+public:
+   DrawTextOptimizer( MyPainter* pPainter ) 
+      : m_fm(pPainter->fontMetrics())
+   { 
+      m_pPainter = pPainter;
+      m_font = m_pPainter->font();
+      m_pen = m_pPainter->pen();
+   }
+   ~DrawTextOptimizer() { end(); }
+   void setFont( const QFont& f )
+   {
+      if ( f!=m_font )
+      {
+         draw();
+         m_font = f;
+         m_fm = m_pPainter->fontMetrics();
+      }
+   }
+   void setPen( const QPen& pen )
+   {
+      if ( pen!=m_pen )
+      {
+         draw();
+         m_pen = pen;
+      }
+   }
+   void drawText( int x, int y, const QString& s )
+   {
+      if ( y!=m_pos.y() || x != m_pos.x()+m_text.length()*m_fm.width("W") )
+      {
+         draw();
+         m_pos = QPoint(x,y);
+      }
+      m_text += s;
+   }
+   void end()
+   {
+      draw();
+   }
+};
 
 void DiffTextWindowData::writeLine(
    MyPainter& p,
@@ -767,6 +841,7 @@ void DiffTextWindowData::writeLine(
    if ( pLineDiff2 != 0 ) changed |= 2;
 
    QColor c = m_pOptionDialog->m_fgColor;
+   p.setPen(c);
    if ( changed == 2 ) {
       c = m_cDiff2;
    } else if ( changed == 1 ) {
@@ -781,28 +856,27 @@ void DiffTextWindowData::writeLine(
    {
       // First calculate the "changed" information for each character.
       int i=0;
+      QString lineString( pld->pLine, pld->size );
       std::vector<UINT8> charChanged( pld->size );
-      if ( pLineDiff1!=0 || pLineDiff2 != 0 )
+      Merger merger( pLineDiff1, pLineDiff2 );
+      while( ! merger.isEndReached() &&  i<pld->size )
       {
-         Merger merger( pLineDiff1, pLineDiff2 );
-         while( ! merger.isEndReached() &&  i<pld->size )
+         if ( i < pld->size )
          {
-            if ( i < pld->size )
-            {
-               charChanged[i] = merger.whatChanged();
-               ++i;
-            }
-            merger.next();
+            charChanged[i] = merger.whatChanged();
+            ++i;
          }
+         merger.next();
       }
 
       QString s=" ";
       // Convert tabs
       int outPos = 0;
 
-      QString lineString( pld->pLine, pld->size );
       int lineLength = m_bWordWrap ? wrapLineOffset+wrapLineLength : lineString.length();
 
+      DrawTextOptimizer dto( &p );
+
       for( i=wrapLineOffset; i<lineLength; ++i )
       {
          int spaces = 1;
@@ -847,40 +921,39 @@ void DiffTextWindowData::writeLine(
                   QColor lightc = diffBgColor;
                   p.fillRect( xOffset + fontWidth*outPos, yOffset,
                            fontWidth*spaces, fontHeight, lightc );
-                  p.setFont(diffFont);
+                  dto.setFont(diffFont);
                }
 
-               p.setPen( c );
+               dto.setPen( c );
                if ( s[0]==' '  &&  c!=m_pOptionDialog->m_fgColor  &&  charChanged[i]!=0 )
                {
                   if ( m_pOptionDialog->m_bShowWhiteSpaceCharacters && m_pOptionDialog->m_bShowWhiteSpace)
                   {
                      p.fillRect( xOffset + fontWidth*outPos, yOffset+fontAscent,
-                                 fontWidth*spaces-1, fontDescent, c );  // QT3
-                                 //fontWidth*spaces-1, fontDescent, c );  // QT4
+                                 fontWidth*spaces-1, fontDescent, c );
                   }
                }
                else
                {
-                  p.drawText( xOffset + fontWidth*outPos, yOffset + fontAscent, s );
+                  dto.drawText( xOffset + fontWidth*outPos, yOffset + fontAscent, s );
                }
-               p.setFont(normalFont);
+               dto.setFont(normalFont);
             }
             else
             {
                p.fillRect( xOffset + fontWidth*outPos, yOffset,
                            fontWidth*(spaces), fontHeight, m_pDiffTextWindow->palette().highlight() );
 
-               p.setPen( m_pDiffTextWindow->palette().highlightedText().color() );
-               p.drawText( xOffset + fontWidth*outPos, yOffset + fontAscent, s );
+               dto.setPen( m_pDiffTextWindow->palette().highlightedText().color() );
+               dto.drawText( xOffset + fontWidth*outPos, yOffset + fontAscent, s );
 
                m_selection.bSelectionContainsData = true;
             }
          }
 
          outPos += spaces;
-      }
-
+      } // end for
+      dto.end();
       if( m_selection.lineWithin( line ) && m_selection.lineWithin( line+1 ) )
       {
          p.fillRect( xOffset + fontWidth*outPos, yOffset,
@@ -947,14 +1020,17 @@ void DiffTextWindowData::writeLine(
 
 void DiffTextWindow::paintEvent( QPaintEvent* e )
 {
-   if ( d->m_pDiff3LineVector==0 || ! d->m_bPaintingAllowed ||
-        ( d->m_diff3WrapLineVector.empty() && d->m_bWordWrap ) ) 
-      return;
-
    QRect invalidRect = e->rect();
-   if ( invalidRect.isEmpty() )
+   if ( invalidRect.isEmpty() || ! d->m_bPaintingAllowed )
       return;
 
+   if ( d->m_pDiff3LineVector==0  ||  ( d->m_diff3WrapLineVector.empty() && d->m_bWordWrap ) ) 
+   {
+      QPainter p(this);
+      p.fillRect( invalidRect, d->m_pOptionDialog->m_bgColor );
+      return;
+   }
+   
    bool bOldSelectionContainsData = d->m_selection.bSelectionContainsData;
    d->m_selection.bSelectionContainsData = false;
 
@@ -1016,7 +1092,7 @@ void DiffTextWindow::print( MyPainter& p, const QRect&, int firstLine, int nofLi
 
 void DiffTextWindowData::draw( MyPainter& p, const QRect& invalidRect, int deviceWidth, int beginLine, int endLine )
 {
-   m_lineNumberWidth =  m_pOptionDialog->m_bShowLineNumbers ? (int)log10((double)m_size)+1 : 0;
+   m_lineNumberWidth = m_pOptionDialog->m_bShowLineNumbers ? (int)log10((double)qMax(m_size,1))+1 : 0;
 
    if ( m_winIdx==1 )
    {
@@ -1185,6 +1261,9 @@ int DiffTextWindow::getNofVisibleColumns()
 
 QString DiffTextWindow::getSelection()
 {
+   if ( d->m_pLineData==0 )
+      return QString();
+
    QString selectionString;
 
    int line=0;
@@ -1234,7 +1313,7 @@ QString DiffTextWindow::getSelection()
             !( d->m_bWordWrap && it+1<vectorSize && d3l == d->m_diff3WrapLineVector[it+1].pD3L ) 
            )
          {
-            #ifdef _WIN32
+            #if defined(_WIN32) || defined(Q_OS_OS2)
             selectionString += '\r';
             #endif
             selectionString += '\n';
@@ -1575,13 +1654,17 @@ public:
    OptionDialog* m_pOptionDialog;
    QLabel*       m_pLabel;
    QLabel*       m_pTopLine;
+   QLabel*       m_pEncoding;
+   QLabel*       m_pLineEndStyle;
    QWidget*      m_pTopLineWidget;
+   int           m_winIdx;
 };
 
 DiffTextWindowFrame::DiffTextWindowFrame( QWidget* pParent, QStatusBar* pStatusBar, OptionDialog* pOptionDialog, int winIdx )
    : QWidget( pParent )
 {
    d = new DiffTextWindowFrameData;
+   d->m_winIdx = winIdx;
    setAutoFillBackground(true);
    d->m_pOptionDialog = pOptionDialog;
    d->m_pTopLineWidget = new QWidget(this);
@@ -1595,8 +1678,17 @@ DiffTextWindowFrame::DiffTextWindowFrame( QWidget* pParent, QStatusBar* pStatusB
    d->m_pTopLine = new QLabel(d->m_pTopLineWidget);
    d->m_pDiffTextWindow = 0;
    d->m_pDiffTextWindow = new DiffTextWindow( this, pStatusBar, pOptionDialog, winIdx );
-   QHBoxLayout* pHL = new QHBoxLayout(d->m_pTopLineWidget);
-   pHL->setMargin(2);
+
+   QVBoxLayout* pVTopLayout = new QVBoxLayout(d->m_pTopLineWidget);
+   pVTopLayout->setMargin(2);
+   pVTopLayout->setSpacing(0);
+   QHBoxLayout* pHL = new QHBoxLayout();
+   QHBoxLayout* pHL2 = new QHBoxLayout();
+   pVTopLayout->addLayout(pHL);
+   pVTopLayout->addLayout(pHL2);
+
+   // Upper line:
+   pHL->setMargin(0);
    pHL->setSpacing(2);
 
    pHL->addWidget( d->m_pLabel, 0 );
@@ -1604,6 +1696,15 @@ DiffTextWindowFrame::DiffTextWindowFrame( QWidget* pParent, QStatusBar* pStatusB
    pHL->addWidget( d->m_pBrowseButton, 0 );
    pHL->addWidget( d->m_pTopLine, 0 );
 
+   // Lower line
+   pHL2->setMargin(0);
+   pHL2->setSpacing(2);
+   pHL2->addWidget( d->m_pTopLine, 0 );
+   d->m_pEncoding = new QLabel(i18n("Encoding:"));
+   d->m_pLineEndStyle = new QLabel(i18n("Line end style:"));
+   pHL2->addWidget(d->m_pEncoding);
+   pHL2->addWidget(d->m_pLineEndStyle);
+
    QVBoxLayout* pVL = new QVBoxLayout( this );
    pVL->setMargin(0);
    pVL->setSpacing(0);
@@ -1632,6 +1733,8 @@ void DiffTextWindowFrame::init()
                              ( pDTW->d->m_bTriple?"A (Base)":"A") :
                              ( pDTW->d->m_winIdx==2 ? "B" : "C" );
       d->m_pLabel->setText( winId + ":" );
+      d->m_pEncoding->setText( i18n("Encoding:") + " " + (pDTW->d->m_pTextCodec!=0 ? pDTW->d->m_pTextCodec->name() : QString()) );
+      d->m_pLineEndStyle->setText( i18n("Line end style:") + " " + (pDTW->d->m_eLineEndStyle==eLineEndStyleDos ? i18n("DOS") : i18n("Unix")) );
    }
 }
 
@@ -1654,7 +1757,7 @@ void DiffTextWindowFrame::setFirstLine( int firstLine )
    if ( pDTW && pDTW->d->m_pDiff3LineVector )
    {
       QString s= i18n("Top line");
-      int lineNumberWidth = (int)log10((double)pDTW->d->m_size)+1;
+      int lineNumberWidth = (int)log10((double)qMax(pDTW->d->m_size,1))+1;
 
       int l=pDTW->calcTopLineInFile(firstLine);
 
@@ -1683,7 +1786,11 @@ bool DiffTextWindowFrame::eventFilter( QObject* o, QEvent* e )
    if ( e->type()==QEvent::FocusIn || e->type()==QEvent::FocusOut )
    {
       QColor c1 = d->m_pOptionDialog->m_bgColor;
-      QColor c2 = pDTW->d->m_cThis;
+      QColor c2;
+      if      ( d->m_winIdx==1 ) c2 = d->m_pOptionDialog->m_colorA;
+      else if ( d->m_winIdx==2 ) c2 = d->m_pOptionDialog->m_colorB;
+      else if ( d->m_winIdx==3 ) c2 = d->m_pOptionDialog->m_colorC;
+
       QPalette p = d->m_pTopLineWidget->palette();
       if ( e->type()==QEvent::FocusOut )
          std::swap(c1,c2);
@@ -1694,11 +1801,13 @@ bool DiffTextWindowFrame::eventFilter( QObject* o, QEvent* e )
       p.setColor(QPalette::WindowText, c1);
       d->m_pLabel->setPalette( p );
       d->m_pTopLine->setPalette( p );
+      d->m_pEncoding->setPalette( p );
+      d->m_pLineEndStyle->setPalette( p );
    }
    if (o == d->m_pFileSelection && e->type()==QEvent::Drop)
    {
       QDropEvent* d = static_cast<QDropEvent*>(e);
-      
+
       if ( d->mimeData()->hasUrls() ) 
       {
          QList<QUrl> lst = d->mimeData()->urls();
@@ -1728,12 +1837,10 @@ void DiffTextWindowFrame::slotBrowseButtonClicked()
 {
    QString current = d->m_pFileSelection->text();
 
-   KURL newURL = KFileDialog::getOpenURL( current, 0, this);
+   KUrl newURL = KFileDialog::getOpenUrl( current, 0, this);
    if ( !newURL.isEmpty() )
    {
       DiffTextWindow* pDTW = d->m_pDiffTextWindow;
       emit fileNameChanged( newURL.url(), pDTW->d->m_winIdx );
    }
 }
-
-//#include "difftextwindow.moc"
