@@ -21,17 +21,17 @@
 
 #include <assert.h>
 
-#include <qnamespace.h>
-#include <qmessagebox.h>
-#include <qmenu.h>
-#include <qmenubar.h>
-#include <qpainter.h>
-#include <qcolordialog.h>
-#include <qfontdialog.h>
-#include <qlabel.h>
-#include <qtextbrowser.h>
-#include <qtextstream.h>
-#include <qlayout.h>
+#include <Qt> //namespace
+#include <QMessageBox>
+#include <QMenu>
+#include <QMenuBar>
+#include <QPainter>
+#include <QColorDialog>
+#include <QFontDialog>
+#include <QLabel>
+#include <QTextBrowser>
+#include <QTextStream>
+#include <QLayout>
 #include <QTabWidget>
 #include <QPaintEvent>
 #include <QPixmap>
@@ -62,13 +62,8 @@ static KAboutData* s_pAboutData;
 
 static void showHelp()
 {
-   #ifdef _WIN32
-      char buf[200];
-      int r= SearchPathA( 0, ".",  0, sizeof(buf), buf, 0 );
-
-      QString exePath;
-      if (r!=0)  {  exePath = buf; }
-      else       {  exePath = "."; }
+   #if defined(_WIN32) || defined(Q_OS_OS2)
+      QString exePath = QCoreApplication::applicationDirPath();
 
       QFileInfo helpFile( exePath + "\\doc\\en\\index.html" );
       if ( ! helpFile.exists() ) {  helpFile.setFile( exePath + "\\..\\doc\\en\\index.html" );     }
@@ -83,24 +78,28 @@ static void showHelp()
          return;
       }
 
+#ifndef Q_OS_OS2
+      char buf[256];
       HINSTANCE hi = FindExecutableA( helpFile.fileName().toAscii(), helpFile.absolutePath().toAscii(), buf );
-      if ( int(hi)<=32 )
+      if ( (quintptr)hi<=32 )
       {
+#endif
          static QTextBrowser* pBrowser = 0;
          if (pBrowser==0)
          {
             pBrowser = new QTextBrowser( 0 );
             pBrowser->setMinimumSize( 600, 400 );
          }
-         pBrowser->setSource(helpFile.filePath());
+         pBrowser->setSource(QUrl("file:///"+helpFile.filePath()));
          pBrowser->show();
+#ifndef Q_OS_OS2
       }
       else
       {
          QFileInfo prog( buf );
          QProcess::startDetached ( prog.filePath(), QStringList( "file:///"+helpFile.absoluteFilePath() ) );
       }
-
+#endif
    #else
       static QTextBrowser* pBrowser = 0;
       if (pBrowser==0)
@@ -115,14 +114,9 @@ static void showHelp()
 
 QString getTranslationDir(const QString& locale)
 {
-   #ifdef _WIN32
-      wchar_t buf[200];
-      int r= SearchPathW( 0, L".",  0, sizeof(buf)/sizeof(wchar_t), buf, 0 );
-
-      buf[ sizeof(buf)/sizeof(wchar_t) -1 ] = 0;
+   #if defined(_WIN32) || defined(Q_OS_OS2)
       QString exePath;
-      if (r!=0)  {  exePath = QString::fromUtf16( (ushort*)&buf[0] ); }
-      else       {  exePath = "."; }
+      exePath = QCoreApplication::applicationDirPath();
       return exePath+"/translations";
    #else
       return  (QString)"/usr/share/locale/" + locale + "/LC_MESSAGES";
@@ -682,7 +676,7 @@ KToggleAction* KStandardAction::showToolbar( QWidget* parent, const char* slot,
 KToggleAction* KStandardAction::showStatusbar( QWidget* parent, const char* slot, KActionCollection* actionCollection)
 {
    KMainWindow* p = actionCollection->m_pMainWindow;
-   KToggleAction* a = new KToggleAction( i18n("Show &Statusbar"), 0, parent, slot, actionCollection, "showstatusbar", false );
+   KToggleAction* a = new KToggleAction( i18n("Show &Status Bar"), 0, parent, slot, actionCollection, "showstatusbar", false );
    if(p) p->settingsMenu->addAction( a );
    return a;
 }
@@ -842,6 +836,7 @@ void KColorButton::slotClicked()
    update();
 }
 
+#ifndef QT_NO_PRINTER
 KPrinter::KPrinter()
 {
 }
@@ -861,7 +856,7 @@ void KPrinter::setCurrentPage(int)
 void KPrinter::setPageSelection(e_PageSelection)
 {
 }
-
+#endif
 
 QPixmap KIconLoader::loadIcon( const QString&, int, int )
 {
@@ -917,7 +912,7 @@ struct KCmdLineOptionsItem
 };
 static QList<KCmdLineOptionsItem> s_options;
 
-static std::vector<QCStringList> s_vOption;
+static std::vector<QStringList> s_vOption;
 static std::vector<QString> s_vArg;
 
 KCmdLineOptions& KCmdLineOptions::add( const QString& name, const QString& description )
@@ -1040,14 +1035,15 @@ KApplication::KApplication()
       ++i;
    }
 
+   QStringList args = QCoreApplication::arguments();
    // First find the option "-config" or "--config" to allow loading of options
    QString configFileName;
-   for( i=1; i<s_argc-1; ++i )
+   for( i=1; i<args.size()-1; ++i )
    {
-      QString arg = s_argv[i];
+      QString arg = args[i];
       if ( arg == "-config" || arg == "--config" )
       {
-         configFileName = s_argv[i+1];
+         configFileName = args[i+1];
       }
    }
    m_config.readConfigFile(configFileName);
@@ -1059,9 +1055,9 @@ KApplication::KApplication()
 
    s_vOption.resize(nofOptions);
 
-   for( i=1; i<s_argc; ++i )
+   for( i=1; i<args.size(); ++i )
    {
-      QString arg = s_argv[i];
+      QString arg = args[i];
       if ( arg[0]=='-' )  // An option
       {
          if ( ignorableCmdLineOptions.contains( arg +";") )
@@ -1075,8 +1071,8 @@ KApplication::KApplication()
             int len = pos==-1 ? optName.length() : pos;
             int len2 = arg.length();
 
-            if( len>0 && ( arg[1]=='-' && len2-2==len && optName.left(len) ==  arg.mid(2,len) ||
-                                          len2-1==len && optName.left(len) ==  arg.mid(1,len) ))
+            if( len>0 && ( (arg[1]=='-' && len2-2==len && optName.left(len) ==  arg.mid(2,len)) ||
+                                          (len2-1==len && optName.left(len) ==  arg.mid(1,len)) ))
             {
                if (s_options[j].description == 0)  // alias, because without description.
                {
@@ -1084,7 +1080,7 @@ KApplication::KApplication()
                   optName = s_options[j].name;
                   pos = optName.indexOf( ' ' );
                }
-               if (pos!=-1){ ++i; s_vOption[j].append( s_argv[i] ); } //use param
+               if (pos!=-1){ ++i; s_vOption[j].append( args[i] ); } //use param
                else        { s_vOption[j].append("1"); }           //set state
                break;
             }
@@ -1136,7 +1132,7 @@ KApplication::KApplication()
             }
 
             s += "\n"+i18n("For more documentation, see the help-menu or the subdirectory doc.")+"\n";
-#ifdef _WIN32
+#if defined(_WIN32) || defined(Q_OS_OS2)
             // A windows program has no console
             if ( 0==QMessageBox::information(0, i18n("KDiff3-Usage"), s, i18n("Ignore"),i18n("Exit") ) )
                continue;
