@@ -27,7 +27,7 @@
 #include <QRadioButton>
 #include <QGroupBox>
 #include <QTextCodec>
-
+#include <QDir>
 #include <QSettings>
 #include <QLocale>
 #include <QGridLayout>
@@ -733,7 +733,7 @@ void OptionDialog::setupEditPage( void )
    gbox->addWidget( pReplaceTabs, line, 0, 1, 2 );
    pReplaceTabs->setToolTip( i18n(
       "On: Pressing tab generates the appropriate number of spaces.\n"
-      "Off: A Tab-character will be inserted.")
+      "Off: A tab character will be inserted.")
       );
    ++line;
 
@@ -964,7 +964,7 @@ void OptionDialog::setupMergePage( void )
       m_pHistoryStartRegExpLineEdit = new OptionLineEdit( ".*\\$Log.*\\$.*", "HistoryStartRegExp", &m_historyStartRegExp, page, this );
       gbox->addWidget( m_pHistoryStartRegExpLineEdit, line, 1 );
       s_historyStartRegExpToolTip = i18n("Regular expression for the start of the version control history entry.\n"
-            "Usually this line contains the \"$Log$\"-keyword.\n"
+            "Usually this line contains the \"$Log$\" keyword.\n"
             "Default value: \".*\\$Log.*\\$.*\"");
       label->setToolTip( s_historyStartRegExpToolTip );
       ++line;
@@ -1005,7 +1005,7 @@ void OptionDialog::setupMergePage( void )
       gbox->addWidget( label, line, 0 );
       m_pHistorySortKeyOrderLineEdit = new OptionLineEdit( defaultSortKeyOrder, "HistoryEntryStartSortKeyOrder", &m_historyEntryStartSortKeyOrder, page, this );
       gbox->addWidget( m_pHistorySortKeyOrderLineEdit, line, 1 );
-      s_historyEntryStartSortKeyOrderToolTip = i18n("Each parentheses used in the regular expression for the history start entry\n"
+      s_historyEntryStartSortKeyOrderToolTip = i18n("Each pair of parentheses used in the regular expression for the history start entry\n"
             "groups a key that can be used for sorting.\n"
             "Specify the list of keys (that are numbered in order of occurrence\n"
             "starting with 1) using ',' as separator (e.g. \"4,5,6,1,2,3,7\").\n"
@@ -1047,9 +1047,9 @@ void OptionDialog::setupMergePage( void )
    OptionCheckBox* pAutoSaveAndQuit = new OptionCheckBox( i18n("Auto save and quit on merge without conflicts"), false,
       "AutoSaveAndQuitOnMergeWithoutConflicts", &m_bAutoSaveAndQuitOnMergeWithoutConflicts, page, this );
    gbox->addWidget( pAutoSaveAndQuit, line, 0, 1, 2 );
-   pAutoSaveAndQuit->setToolTip( i18n("When KDiff3 was started for a file-merge from the commandline and all\n" 
+   pAutoSaveAndQuit->setToolTip( i18n("If KDiff3 was started for a file-merge from the command line and all\n"
                                          "conflicts are solvable without user interaction then automatically save and quit.\n"
-                                         "(Similar to command line option \"--auto\".") );
+                                         "(Similar to command line option \"--auto\".)") );
    ++line;
 
    topLayout->addStretch(10);
@@ -1113,13 +1113,13 @@ void OptionDialog::setupDirectoryMergePage( void )
    gbox->addWidget( pUseCvsIgnore, line, 0, 1, 2 );
    pUseCvsIgnore->setToolTip( i18n(
       "Extends the antipattern to anything that would be ignored by CVS.\n"
-      "Via local \".cvsignore\"-files this can be directory specific."
+      "Via local \".cvsignore\" files this can be directory specific."
       ));
    ++line;
 
    OptionCheckBox* pFindHidden = new OptionCheckBox( i18n("Find hidden files and directories"), true, "FindHidden", &m_bDmFindHidden, page, this );
    gbox->addWidget( pFindHidden, line, 0, 1, 2 );
-#ifdef _WIN32
+#if defined(_WIN32) || defined(Q_OS_OS2)
    pFindHidden->setToolTip( i18n("Finds files and directories with the hidden attribute.") );
 #else
    pFindHidden->setToolTip( i18n("Finds files and directories starting with '.'.") );
@@ -1148,7 +1148,7 @@ void OptionDialog::setupDirectoryMergePage( void )
    //              "Files and directories without change will not appear in the list."));
    //++line;
 
-#ifdef _WIN32
+#if defined(_WIN32) || defined(Q_OS_OS2)
    bool bCaseSensitiveFilenameComparison = false;
 #else
    bool bCaseSensitiveFilenameComparison = true;
@@ -1181,9 +1181,9 @@ void OptionDialog::setupDirectoryMergePage( void )
                                      "Useful for big directories or slow networks.") );
    pBGLayout->addWidget( pTrustDate );
                                      
-   OptionRadioButton* pTrustDateFallbackToBinary = new OptionRadioButton( i18n("Trust the size and date, but use binary comparison if date doesn't match (unsafe)"), false, "TrustDateFallbackToBinary", &m_bDmTrustDateFallbackToBinary, pBG, this );
+   OptionRadioButton* pTrustDateFallbackToBinary = new OptionRadioButton( i18n("Trust the size and date, but use binary comparison if date does not match (unsafe)"), false, "TrustDateFallbackToBinary", &m_bDmTrustDateFallbackToBinary, pBG, this );
    pTrustDateFallbackToBinary->setToolTip( i18n("Assume that files are equal if the modification date and file length are equal.\n"
-                                     "If the date isn't equal but the sizes are, use binary comparison.\n"
+                                     "If the dates are not equal but the sizes are, use binary comparison.\n"
                                      "Useful for big directories or slow networks.") );
    pBGLayout->addWidget( pTrustDateFallbackToBinary );
 
@@ -1225,8 +1225,8 @@ void OptionDialog::setupDirectoryMergePage( void )
    OptionCheckBox* pCreateBakFiles = new OptionCheckBox( i18n("Backup files (.orig)"), true, "CreateBakFiles", &m_bDmCreateBakFiles, page, this );
    gbox->addWidget( pCreateBakFiles, line, 0, 1, 2 );
    pCreateBakFiles->setToolTip( i18n(
-                 "When a file would be saved over an old file, then the old file\n"
-                 "will be renamed with a '.orig'-extension instead of being deleted."));
+                 "If a file would be saved over an old file, then the old file\n"
+                 "will be renamed with a '.orig' extension instead of being deleted."));
    ++line;
 
    topLayout->addStretch(10);
@@ -1254,65 +1254,42 @@ static void insertCodecs(OptionComboBox* p)
 // UTF8-Codec that saves a BOM
 class Utf8BOMCodec : public QTextCodec
 {
+   QTextCodec* m_pUtf8Codec;
+   class PublicTextCodec : public QTextCodec
+   {
    public:
+      QString publicConvertToUnicode ( const char * p, int len, ConverterState* pState ) const
+      {
+         return convertToUnicode( p, len, pState );
+      }
+      QByteArray publicConvertFromUnicode ( const QChar * input, int number, ConverterState * pState ) const
+      {
+         return convertFromUnicode( input, number, pState );
+      }
+   };
+public:
+   Utf8BOMCodec()
+   {
+      m_pUtf8Codec = QTextCodec::codecForName("UTF-8");
+   }
    QByteArray name () const { return "UTF-8-BOM"; }
    int mibEnum () const { return 2123; }
    QByteArray convertFromUnicode ( const QChar * input, int number, ConverterState * pState ) const
    {
       QByteArray r;
-      if ( pState && pState->state_data[0]==0)
+      if ( pState && pState->state_data[2]==0)  // state_data[2] not used by QUtf8::convertFromUnicode (see qutfcodec.cpp)
       {
         r += "\xEF\xBB\xBF";
-        pState->state_data[0]=1;
+        pState->state_data[2]=1;
+        pState->flags |= QTextCodec::IgnoreHeader;
       }
 
-      r += QString::fromRawData( input, number ).toUtf8();
+      r += ((PublicTextCodec*)m_pUtf8Codec)->publicConvertFromUnicode( input, number, pState );
       return r;
    }
-   QString convertToUnicode ( const char * p, int len, ConverterState* ) const
+   QString convertToUnicode ( const char * p, int len, ConverterState* pState ) const
    {
-      return QString::fromUtf8 ( p, len );
-   }
-
-   class UTF8BOMEncoder : public QTextEncoder
-   {
-      bool bBOMAdded;
-   public:
-      UTF8BOMEncoder(const QTextCodec* pTC):QTextEncoder(pTC)  {  bBOMAdded=false;  }
-      QByteArray fromUnicode(const QString& uc, int& lenInOut )
-      {
-         QByteArray r;
-         if (!bBOMAdded)
-         {
-            r += "\xEF\xBB\xBF";
-            bBOMAdded=true;
-         }
-         r += uc.toUtf8();
-         lenInOut = r.length();
-         return r;
-      }
-   };
-   QTextEncoder* makeEncoder() const
-   {
-      return new UTF8BOMEncoder(this);
-   }
-
-   class UTF8BOMDecoder : public QTextDecoder
-   {
-      QTextDecoder *m_pDecoder;
-   public:
-      UTF8BOMDecoder(const QTextCodec* pTC) : QTextDecoder(pTC)  {  m_pDecoder = QTextCodec::codecForName("UTF-8")->makeDecoder();  }
-      ~UTF8BOMDecoder() {
-         delete m_pDecoder;
-      }
-      QString toUnicode( const char* p, int len)
-      {
-         return m_pDecoder->toUnicode( p, len );
-      }
-   };
-   QTextDecoder* makeDecoder() const
-   {
-      return new UTF8BOMDecoder(this);
+      return ((PublicTextCodec*)m_pUtf8Codec)->publicConvertToUnicode( p, len, pState );
    }
 };
 
@@ -1338,7 +1315,7 @@ void OptionDialog::setupRegionalPage( void )
    QLabel* label;
 
 #ifdef KREPLACEMENTS_H
-
+ 
 static const char* countryMap[]={
 "af Afrikaans",
 "ar Arabic",
@@ -1350,6 +1327,7 @@ static const char* countryMap[]={
 "br Breton",
 "bs Bosnian",
 "ca Catalan",
+"ca@valencia Catalan (Valencian)",
 "cs Czech",
 "cy Welsh",
 "da Danish",
@@ -1369,6 +1347,7 @@ static const char* countryMap[]={
 "gu Gujarati",
 "he Hebrew",
 "hi Hindi",
+"hne Chhattisgarhi",
 "hr Croatian",
 "hsb Upper Sorbian",
 "hu Hungarian",
@@ -1384,6 +1363,7 @@ static const char* countryMap[]={
 "lv Latvian",
 "mi Maori",
 "mk Macedonian",
+"ml Malayalam"
 "mn Mongolian",
 "ms Malay",
 "mt Maltese",
@@ -1429,7 +1409,7 @@ static const char* countryMap[]={
    pLanguage->addItem( "Auto" );  // Must not translate, won't work otherwise!
    pLanguage->addItem( "en_orig" );
       
-#ifndef _WIN32
+#if !defined(_WIN32) && !defined(Q_OS_OS2) 
    // Read directory: Find all kdiff3_*.qm-files and insert the found files here
    QDir localeDir( "/usr/share/locale" ); // See also kreplacements.cpp: getTranslationDir()
    QStringList dirList = localeDir.entryList(QDir::Dirs | QDir::NoDotAndDotDot, QDir::Name);
@@ -1464,9 +1444,8 @@ static const char* countryMap[]={
       pLanguage->addItem( languageId );
    }
    
-   
    label->setToolTip( i18n(
-      "Choose the language of the GUI-strings or \"Auto\".\n" 
+      "Choose the language of the GUI strings or \"Auto\".\n"
       "For a change of language to take place, quit and restart KDiff3.") 
       );
    ++line;
@@ -1591,7 +1570,7 @@ void OptionDialog::setupIntegrationPage( void )
    label->setToolTip( i18n(
       "List of command line options that should be ignored when KDiff3 is used by other tools.\n"
       "Several values can be specified if separated via ';'\n"
-      "This will suppress the \"Unknown option\"-error."
+      "This will suppress the \"Unknown option\" error."
       ));
    ++line;
 
@@ -1600,7 +1579,7 @@ void OptionDialog::setupIntegrationPage( void )
    gbox->addWidget( pEscapeKeyQuits, line, 0, 1, 2 );
    pEscapeKeyQuits->setToolTip( i18n(
                   "Fast method to exit.\n"
-                  "For those who are used to using the Escape-key."  ) );
+                  "For those who are used to using the Escape key."  ) );
    ++line;
 
 #ifdef _WIN32
@@ -1608,7 +1587,7 @@ void OptionDialog::setupIntegrationPage( void )
    gbox->addWidget( pIntegrateWithClearCase, line, 0 );
    pIntegrateWithClearCase->setToolTip( i18n(
                  "Integrate with Rational ClearCase from IBM.\n"
-                 "Modifies the \"map\" file in ClearCase-subdir \"lib/mgrs\"\n"
+                 "Modifies the \"map\" file in ClearCase subdir \"lib/mgrs\"\n"
                  "(Only enabled when ClearCase \"bin\" directory is in the path.)"));
    connect(pIntegrateWithClearCase, SIGNAL(clicked()),this, SLOT(slotIntegrateWithClearCase()) );
    pIntegrateWithClearCase->setEnabled( integrateWithClearCase( "existsClearCase", "" )!=0 );
@@ -1616,7 +1595,7 @@ void OptionDialog::setupIntegrationPage( void )
    QPushButton* pRemoveClearCaseIntegration = new QPushButton( i18n("Remove ClearCase Integration"), page);
    gbox->addWidget( pRemoveClearCaseIntegration, line, 1 );
    pRemoveClearCaseIntegration->setToolTip( i18n(
-                 "Restore the old \"map\" file from before doing the Clearcase integration."));
+                 "Restore the old \"map\" file from before doing the ClearCase integration."));
    connect(pRemoveClearCaseIntegration, SIGNAL(clicked()),this, SLOT(slotRemoveClearCaseIntegration()) );
    pRemoveClearCaseIntegration->setEnabled( integrateWithClearCase( "existsClearCase", "" )!=0 );
 
