@@ -24,15 +24,15 @@
 #include <algorithm>
 #include <ctype.h>
 
-#include <qclipboard.h>
-#include <qscrollbar.h>
-#include <qlayout.h>
-#include <qcheckbox.h>
-#include <qsplitter.h>
-#include <qdir.h>
-#include <qfile.h>
-#include <qlineedit.h>
-#include <qcombobox.h>
+#include <QClipboard>
+#include <QScrollBar>
+#include <QLayout>
+#include <QCheckBox>
+#include <QSplitter>
+#include <QDir>
+#include <QFile>
+#include <QLineEdit>
+#include <QComboBox>
 #include <QDropEvent>
 #include <QUrl>
 #include <QProcess>
@@ -41,7 +41,6 @@
 #include <kmessagebox.h>
 #include <kfontdialog.h>
 #include <kstatusbar.h>
-#include <kkeydialog.h>
 
 
 #include <assert.h>
@@ -53,6 +52,7 @@
 #include <windows.h>
 #else
 #include <unistd.h>
+#include <kshortcutsdialog.h>
 #endif
 
 #include "gnudiff_diff.h"
@@ -266,7 +266,8 @@ void KDiff3App::init( bool bAuto, TotalDiffStatus* pTotalDiffStatus, bool bLoadF
                                i18n("\nMost likely this is not wanted during a merge.\n"
                                     "Do you want to disable these settings or continue with these settings active?"),
                                i18n("Option Unsafe for Merging"), 
-                               i18n("Use These Options During Merge"), i18n("Disable Unsafe Options")
+                               KGuiItem( i18n("Use These Options During Merge") ), 
+                               KGuiItem( i18n("Disable Unsafe Options") )
                                );
 
          if (result == KMessageBox::No )
@@ -358,9 +359,12 @@ void KDiff3App::init( bool bAuto, TotalDiffStatus* pTotalDiffStatus, bool bLoadF
       correctManualDiffAlignment( m_diff3LineList, &m_manualDiffHelpList );
       calcDiff3LineListTrim( m_diff3LineList, m_sd1.getLineDataForDiff(), m_sd2.getLineDataForDiff(), m_sd3.getLineDataForDiff(), &m_manualDiffHelpList );
 
-      calcDiff3LineListUsingBC( &m_diffList23, m_diff3LineList );
-      correctManualDiffAlignment( m_diff3LineList, &m_manualDiffHelpList );
-      calcDiff3LineListTrim( m_diff3LineList, m_sd1.getLineDataForDiff(), m_sd2.getLineDataForDiff(), m_sd3.getLineDataForDiff(), &m_manualDiffHelpList );
+      if ( m_pOptionDialog->m_bDiff3AlignBC )
+      {
+         calcDiff3LineListUsingBC( &m_diffList23, m_diff3LineList );
+         correctManualDiffAlignment( m_diff3LineList, &m_manualDiffHelpList );
+         calcDiff3LineListTrim( m_diff3LineList, m_sd1.getLineDataForDiff(), m_sd2.getLineDataForDiff(), m_sd3.getLineDataForDiff(), &m_manualDiffHelpList );
+      }
       debugLineCheck( m_diff3LineList, m_sd1.getSizeLines(), 1 );
       debugLineCheck( m_diff3LineList, m_sd2.getSizeLines(), 2 );
       debugLineCheck( m_diff3LineList, m_sd3.getSizeLines(), 3 );
@@ -387,8 +391,6 @@ void KDiff3App::init( bool bAuto, TotalDiffStatus* pTotalDiffStatus, bool bLoadF
    // Calc needed lines for display
    m_neededLines = m_diff3LineList.size();
 
-   m_pDirectoryMergeWindow->allowResizeEvents(false);
-
    QList<int> oldHeights;
    if ( m_pDirectoryMergeSplitter->isVisible() )
       oldHeights = m_pMainSplitter->sizes();
@@ -404,11 +406,15 @@ void KDiff3App::init( bool bAuto, TotalDiffStatus* pTotalDiffStatus, bool bLoadF
          oldHeights[1] = oldHeights[0]/2;
          oldHeights[0] -= oldHeights[1];
       }
+      if ( oldHeights[0]==0 && oldHeights[1]==0 )
+      {
+         oldHeights[1] = 100;
+         oldHeights[0] = 100;
+      }
       m_pMainSplitter->setSizes( oldHeights );
    }
 
    m_pMainWidget->setVisible( bGUI );
-   m_pDirectoryMergeWindow->allowResizeEvents(true);
 
    m_bTripleDiff = ! m_sd3.isEmpty();
 
@@ -416,14 +422,16 @@ void KDiff3App::init( bool bAuto, TotalDiffStatus* pTotalDiffStatus, bool bLoadF
    if ( ! m_pOptionDialog->m_bAutoSelectOutEncoding )
       m_pMergeResultWindowTitle->setEncoding( m_pOptionDialog->m_pEncodingOut );
 
+   m_pMergeResultWindowTitle->setLineEndStyles( m_sd1.getLineEndStyle(), m_sd2.getLineEndStyle(), m_sd3.getLineEndStyle() );
+
    if ( bGUI )
    {
       const ManualDiffHelpList* pMDHL = &m_manualDiffHelpList;
-      m_pDiffTextWindow1->init( m_sd1.getAliasName(),
+      m_pDiffTextWindow1->init( m_sd1.getAliasName(), m_sd1.getEncoding(), m_sd1.getLineEndStyle(),
          m_sd1.getLineDataForDisplay(), m_sd1.getSizeLines(), &m_diff3LineVector, pMDHL, m_bTripleDiff );
-      m_pDiffTextWindow2->init( m_sd2.getAliasName(),
+      m_pDiffTextWindow2->init( m_sd2.getAliasName(), m_sd2.getEncoding(), m_sd2.getLineEndStyle(),
          m_sd2.getLineDataForDisplay(), m_sd2.getSizeLines(), &m_diff3LineVector, pMDHL, m_bTripleDiff );
-      m_pDiffTextWindow3->init( m_sd3.getAliasName(),
+      m_pDiffTextWindow3->init( m_sd3.getAliasName(), m_sd3.getEncoding(), m_sd3.getLineEndStyle(),
          m_sd3.getLineDataForDisplay(), m_sd3.getSizeLines(), &m_diff3LineVector, pMDHL, m_bTripleDiff );
 
       m_pDiffTextWindowFrame3->setVisible(m_bTripleDiff);
@@ -466,10 +474,7 @@ void KDiff3App::init( bool bAuto, TotalDiffStatus* pTotalDiffStatus, bool bLoadF
    else
       m_pMergeWindowFrame->show();
 
-   // Calc max width for display
-   m_maxWidth = max2( m_pDiffTextWindow1->getNofColumns(), m_pDiffTextWindow2->getNofColumns() );
-   m_maxWidth = max2( m_maxWidth, m_pDiffTextWindow3->getNofColumns() );
-   m_maxWidth += 5;
+   setHScrollBarRange();
 
    // Try to create a meaningful but not too long caption
    if ( !isPart() )
@@ -528,12 +533,12 @@ void KDiff3App::init( bool bAuto, TotalDiffStatus* pTotalDiffStatus, bool bLoadF
          else  if ( pTotalDiffStatus->bTextAEqB && pTotalDiffStatus->bTextAEqC )
             totalInfo += i18n("All input files contain the same text, but are not binary equal.");
          else {
-            if    ( pTotalDiffStatus->bBinaryAEqB ) totalInfo += i18n("Files %1 and %2 are binary equal.\n"                          ).arg("A").arg("B");
-            else if ( pTotalDiffStatus->bTextAEqB ) totalInfo += i18n("Files %1 and %2 have equal text, but are not binary equal. \n").arg("A").arg("B");
-            if    ( pTotalDiffStatus->bBinaryAEqC ) totalInfo += i18n("Files %1 and %2 are binary equal.\n"                          ).arg("A").arg("C");
-            else if ( pTotalDiffStatus->bTextAEqC ) totalInfo += i18n("Files %1 and %2 have equal text, but are not binary equal. \n").arg("A").arg("C");
-            if    ( pTotalDiffStatus->bBinaryBEqC ) totalInfo += i18n("Files %1 and %2 are binary equal.\n"                          ).arg("B").arg("C");
-            else if ( pTotalDiffStatus->bTextBEqC ) totalInfo += i18n("Files %1 and %2 have equal text, but are not binary equal. \n").arg("B").arg("C");
+            if    ( pTotalDiffStatus->bBinaryAEqB ) totalInfo += i18n("Files %1 and %2 are binary equal.\n",QString("A"),QString("B"));
+            else if ( pTotalDiffStatus->bTextAEqB ) totalInfo += i18n("Files %1 and %2 have equal text, but are not binary equal. \n",QString("A"),QString("B"));
+            if    ( pTotalDiffStatus->bBinaryAEqC ) totalInfo += i18n("Files %1 and %2 are binary equal.\n",QString("A"),QString("C"));
+            else if ( pTotalDiffStatus->bTextAEqC ) totalInfo += i18n("Files %1 and %2 have equal text, but are not binary equal. \n",QString("A"),QString("C"));
+            if    ( pTotalDiffStatus->bBinaryBEqC ) totalInfo += i18n("Files %1 and %2 are binary equal.\n",QString("B"),QString("C"));
+            else if ( pTotalDiffStatus->bTextBEqC ) totalInfo += i18n("Files %1 and %2 have equal text, but are not binary equal. \n",QString("B"),QString("C"));
          }
 
          if ( !totalInfo.isEmpty() )
@@ -547,6 +552,22 @@ void KDiff3App::init( bool bAuto, TotalDiffStatus* pTotalDiffStatus, bool bLoadF
             "Note that the KDiff3-merge was not meant for binary data.\n"
             "Continue at your own risk.") );
       }
+      if ( m_sd1.isIncompleteConversion() || m_sd2.isIncompleteConversion() || m_sd3.isIncompleteConversion() )
+      {
+         QString files;
+         if ( m_sd1.isIncompleteConversion() )
+            files += "A";
+         if ( m_sd2.isIncompleteConversion() )
+            files += files.isEmpty() ? "B" : ", B";
+         if ( m_sd3.isIncompleteConversion() )
+            files += files.isEmpty() ? "C" : ", C";
+            
+         KMessageBox::information( this, i18n(
+            "Some input characters could not be converted to valid unicode.\n"
+            "You might be using the wrong codec. (e.g. UTF-8 for non UTF-8 files).\n"
+            "Don't save the result if unsure. Continue at your own risk.\n"
+            "Affected input files are in %1.").arg(files) );
+      }
    }
 
    QTimer::singleShot( 10, this, SLOT(slotAfterFirstPaint()) );
@@ -562,7 +583,45 @@ void KDiff3App::init( bool bAuto, TotalDiffStatus* pTotalDiffStatus, bool bLoadF
 }
 
 
-void KDiff3App::resizeDiffTextWindow(int newWidth, int newHeight)
+void KDiff3App::setHScrollBarRange()
+{
+   int w1 = m_pDiffTextWindow1!=0 && m_pDiffTextWindow1->isVisible() ? m_pDiffTextWindow1->getNofColumns() : 0;
+   int w2 = m_pDiffTextWindow2!=0 && m_pDiffTextWindow2->isVisible() ? m_pDiffTextWindow2->getNofColumns() : 0;
+   int w3 = m_pDiffTextWindow3!=0 && m_pDiffTextWindow3->isVisible() ? m_pDiffTextWindow3->getNofColumns() : 0;
+
+   int wm = m_pMergeResultWindow!=0 && m_pMergeResultWindow->isVisible() ? m_pMergeResultWindow->getNofColumns() : 0;
+
+   int v1 = m_pDiffTextWindow1!=0 && m_pDiffTextWindow1->isVisible() ? m_pDiffTextWindow1->getNofVisibleColumns() : 0;
+   int v2 = m_pDiffTextWindow2!=0 && m_pDiffTextWindow2->isVisible() ? m_pDiffTextWindow2->getNofVisibleColumns() : 0;
+   int v3 = m_pDiffTextWindow3!=0 && m_pDiffTextWindow3->isVisible() ? m_pDiffTextWindow3->getNofVisibleColumns() : 0;
+   int vm = m_pMergeResultWindow!=0 && m_pMergeResultWindow->isVisible() ? m_pMergeResultWindow->getNofVisibleColumns() : 0;
+
+   // Find the minimum, but don't consider 0.
+   int pageStep = 0;
+   if ( (pageStep==0 || pageStep>v1) && v1>0 )
+      pageStep = v1;
+   if ( (pageStep==0 || pageStep>v2) && v2>0  )
+      pageStep = v2;
+   if ( (pageStep==0 || pageStep>v3) && v3>0  )
+      pageStep = v3;
+   if ( (pageStep==0 || pageStep>vm) && vm>0  )
+      pageStep = vm;
+
+   int rangeMax = 0;
+   if ( w1>v1 && w1-v1>rangeMax && v1>0 )
+      rangeMax = w1-v1;
+   if ( w2>v2 && w2-v2>rangeMax && v2>0 )
+      rangeMax = w2-v2;
+   if ( w3>v3 && w3-v3>rangeMax && v3>0 )
+      rangeMax = w3-v3;
+   if ( wm>vm && wm-vm>rangeMax && vm>0 )
+      rangeMax = wm-vm;
+
+   m_pHScrollBar->setRange(0, rangeMax );
+   m_pHScrollBar->setPageStep( pageStep );
+}
+
+void KDiff3App::resizeDiffTextWindow(int /*newWidth*/, int newHeight)
 {
    m_DTWHeight = newHeight;
 
@@ -572,10 +631,7 @@ void KDiff3App::resizeDiffTextWindow(int newWidth, int newHeight)
    m_pDiffVScrollBar->setPageStep( newHeight );
    m_pOverview->setRange( m_pDiffVScrollBar->value(), m_pDiffVScrollBar->pageStep() );
 
-   // The second window has a somewhat inverse width
-
-   m_pHScrollBar->setRange(0, max2(0, m_maxWidth - newWidth) );
-   m_pHScrollBar->setPageStep( newWidth );
+   setHScrollBarRange();
 }
 
 void KDiff3App::resizeMergeResultWindow()
@@ -584,9 +640,7 @@ void KDiff3App::resizeMergeResultWindow()
    m_pMergeVScrollBar->setRange(0, max2(0, p->getNofLines() - p->getNofVisibleLines()) );
    m_pMergeVScrollBar->setPageStep( p->getNofVisibleLines() );
 
-   // The second window has a somewhat inverse width
-//   m_pHScrollBar->setRange(0, max2(0, m_maxWidth - newWidth) );
-//   m_pHScrollBar->setPageStep( newWidth );
+   setHScrollBarRange();
 }
 
 void KDiff3App::scrollDiffTextWindow( int deltaX, int deltaY )
@@ -615,9 +669,15 @@ void KDiff3App::setDiff3Line( int line )
 
 void KDiff3App::sourceMask( int srcMask, int enabledMask )
 {
+   chooseA->blockSignals(true);
+   chooseB->blockSignals(true);
+   chooseC->blockSignals(true);
    chooseA->setChecked( (srcMask & 1) != 0 );
    chooseB->setChecked( (srcMask & 2) != 0 );
    chooseC->setChecked( (srcMask & 4) != 0 );
+   chooseA->blockSignals(false);
+   chooseB->blockSignals(false);
+   chooseC->blockSignals(false);
    chooseA->setEnabled( (enabledMask & 1) != 0 );
    chooseB->setEnabled( (enabledMask & 2) != 0 );
    chooseC->setEnabled( (enabledMask & 4) != 0 );
@@ -643,30 +703,38 @@ void KDiff3App::initView()
       return;
       //delete m_pMainWidget;
    }
-   m_pMainWidget = new QWidget(m_pMainSplitter); // Contains vertical splitter and horiz scrollbar
+   m_pMainWidget = new QWidget(); // Contains vertical splitter and horiz scrollbar   
+   m_pMainSplitter->addWidget( m_pMainWidget );
+   m_pMainWidget->setObjectName("MainWidget");
    QVBoxLayout* pVLayout = new QVBoxLayout(m_pMainWidget);
    pVLayout->setMargin(0);
    pVLayout->setSpacing(0);
 
    QSplitter* pVSplitter = new QSplitter();
+   pVSplitter->setObjectName("VSplitter");
    pVSplitter->setOpaqueResize(false);
    pVSplitter->setOrientation( Qt::Vertical );
    pVLayout->addWidget( pVSplitter );
    pVSplitter->show();
 
-   QWidget* pDiffWindowFrame = new QWidget( pVSplitter ); // Contains diff windows, overview and vert scrollbar
+   QWidget* pDiffWindowFrame = new QWidget(); // Contains diff windows, overview and vert scrollbar
+   pDiffWindowFrame->setObjectName("DiffWindowFrame");
    QHBoxLayout* pDiffHLayout = new QHBoxLayout( pDiffWindowFrame );
    pDiffHLayout->setMargin(0);
    pDiffHLayout->setSpacing(0);
-   pDiffWindowFrame->show();
+   //pDiffWindowFrame->show();
+   pVSplitter->addWidget(pDiffWindowFrame);
+
    m_pDiffWindowSplitter = new QSplitter();
+   m_pDiffWindowSplitter->setObjectName("DiffWindowSplitter");
    m_pDiffWindowSplitter->setOpaqueResize(false);
 
    m_pDiffWindowSplitter->setOrientation( m_pOptionDialog->m_bHorizDiffWindowSplitting ?  Qt::Horizontal : Qt::Vertical );
    pDiffHLayout->addWidget( m_pDiffWindowSplitter );
-   m_pDiffWindowSplitter->show();
+   //m_pDiffWindowSplitter->show();
 
    m_pOverview = new Overview( m_pOptionDialog );
+   m_pOverview->setObjectName("Overview");
    pDiffHLayout->addWidget(m_pOverview);
    connect( m_pOverview, SIGNAL(setLine(int)), this, SLOT(setDiff3Line(int)) );
    //connect( m_pOverview, SIGNAL(afterFirstPaint()), this, SLOT(slotAfterFirstPaint()));
@@ -675,10 +743,13 @@ void KDiff3App::initView()
    pDiffHLayout->addWidget( m_pDiffVScrollBar );
 
    m_pDiffTextWindowFrame1 = new DiffTextWindowFrame( m_pDiffWindowSplitter, statusBar(), m_pOptionDialog, 1 );
-   m_pDiffTextWindowFrame1->show();
+   m_pDiffWindowSplitter->addWidget(m_pDiffTextWindowFrame1);
+   //m_pDiffTextWindowFrame1->show();
    m_pDiffTextWindowFrame2 = new DiffTextWindowFrame( m_pDiffWindowSplitter, statusBar(), m_pOptionDialog, 2 );
-   m_pDiffTextWindowFrame2->show();
+   m_pDiffWindowSplitter->addWidget(m_pDiffTextWindowFrame2);
+   //m_pDiffTextWindowFrame2->show();
    m_pDiffTextWindowFrame3 = new DiffTextWindowFrame( m_pDiffWindowSplitter, statusBar(), m_pOptionDialog, 3 );
+   m_pDiffWindowSplitter->addWidget(m_pDiffTextWindowFrame3);
    m_pDiffTextWindow1 = m_pDiffTextWindowFrame1->getDiffTextWindow();
    m_pDiffTextWindow2 = m_pDiffTextWindowFrame2->getDiffTextWindow();
    m_pDiffTextWindow3 = m_pDiffTextWindowFrame3->getDiffTextWindow();
@@ -688,6 +759,8 @@ void KDiff3App::initView()
 
    // Merge window
    m_pMergeWindowFrame = new QWidget( pVSplitter );
+   m_pMergeWindowFrame->setObjectName("MergeWindowFrame");
+   pVSplitter->addWidget(m_pMergeWindowFrame);
    QHBoxLayout* pMergeHLayout = new QHBoxLayout( m_pMergeWindowFrame );
    pMergeHLayout->setMargin(0);
    pMergeHLayout->setSpacing(0);
@@ -703,13 +776,21 @@ void KDiff3App::initView()
    m_pMergeVScrollBar = new QScrollBar( Qt::Vertical, m_pMergeWindowFrame );
    pMergeHLayout->addWidget( m_pMergeVScrollBar );
 
+   m_pMainSplitter->addWidget(m_pMainWidget);
+
    autoAdvance->setEnabled(true);
 
    QList<int> sizes = pVSplitter->sizes();
    int total = sizes[0] + sizes[1];
+   if ( total<10 )
+      total = 100;
    sizes[0]=total/2; sizes[1]=total/2;
    pVSplitter->setSizes( sizes );
 
+   QList<int> hSizes;
+   hSizes << 1 << 1 << 1;
+   m_pDiffWindowSplitter->setSizes( hSizes );
+   
    m_pMergeResultWindow->installEventFilter( this );       // for Cut/Copy/Paste-shortcuts
    m_pMergeResultWindow->installEventFilter( m_pMergeResultWindowTitle ); // for focus tracking
 
@@ -791,9 +872,9 @@ static int calcManualDiffFirstDiff3LineIdx( const Diff3LineVector& d3lv, const M
    for( i = 0; i<d3lv.size(); ++i )
    {
       const Diff3Line& d3l = *d3lv[i];
-      if ( mdhe.lineA1>=0  &&  mdhe.lineA1==d3l.lineA ||
-           mdhe.lineB1>=0  &&  mdhe.lineB1==d3l.lineB ||
-           mdhe.lineC1>=0  &&  mdhe.lineC1==d3l.lineC )
+      if ( (mdhe.lineA1>=0  &&  mdhe.lineA1==d3l.lineA) ||
+           (mdhe.lineB1>=0  &&  mdhe.lineB1==d3l.lineB) ||
+           (mdhe.lineC1>=0  &&  mdhe.lineC1==d3l.lineC) )
          return i;
    }
    return -1;
@@ -802,7 +883,7 @@ static int calcManualDiffFirstDiff3LineIdx( const Diff3LineVector& d3lv, const M
 void KDiff3App::slotAfterFirstPaint()
 {
    int newHeight = m_pDiffTextWindow1->getNofVisibleLines();
-   int newWidth  = m_pDiffTextWindow1->getNofVisibleColumns();
+   /*int newWidth  = m_pDiffTextWindow1->getNofVisibleColumns();*/
    m_DTWHeight = newHeight;
 
    recalcWordWrap();
@@ -811,10 +892,6 @@ void KDiff3App::slotAfterFirstPaint()
    m_pDiffVScrollBar->setPageStep( newHeight );
    m_pOverview->setRange( m_pDiffVScrollBar->value(), m_pDiffVScrollBar->pageStep() );
 
-   // The second window has a somewhat inverse width
-   m_pHScrollBar->setRange(0, max2(0, m_maxWidth - newWidth) );
-   m_pHScrollBar->setPageStep( newWidth );
-
    int d3l=-1;
    if ( ! m_manualDiffHelpList.empty() )
       d3l = calcManualDiffFirstDiff3LineIdx( m_diff3LineVector, m_manualDiffHelpList.front() );
@@ -834,6 +911,13 @@ void KDiff3App::slotAfterFirstPaint()
       m_pCornerWidget->setFixedSize( m_pDiffVScrollBar->width(), m_pHScrollBar->height() );
 
    slotUpdateAvailabilities();
+   setUpdatesEnabled(true);
+   // Workaround for a Qt-bug
+   QList<QTreeView*> treeViews = findChildren<QTreeView*>();
+   foreach( QTreeView* pTreeView, treeViews )
+   {
+      pTreeView->setUpdatesEnabled(true);
+   }
 }
 
 void KDiff3App::resizeEvent(QResizeEvent* e)
@@ -866,6 +950,11 @@ bool KDiff3App::eventFilter( QObject* o, QEvent* e )
             slotEditCut();
             return true;
          }
+         if ( k->key()==Qt::Key_Escape && m_pKDiff3Shell && m_pOptionDialog->m_bEscapeKeyQuits )
+         {
+            m_pKDiff3Shell->close();
+            return true;
+         }
       }
       return QSplitter::eventFilter( o, e );    // standard event processing
    }
@@ -873,6 +962,11 @@ bool KDiff3App::eventFilter( QObject* o, QEvent* e )
    if ( e->type() == QEvent::KeyPress )   // key press
    {
        QKeyEvent *k = (QKeyEvent*)e;
+       if ( k->key()==Qt::Key_Escape && m_pKDiff3Shell && m_pOptionDialog->m_bEscapeKeyQuits )
+       {
+          m_pKDiff3Shell->close();
+          return true;
+       }
 
        bool bCtrl = (k->QInputEvent::modifiers() & Qt::ControlModifier) != 0;
        if (k->key()==Qt::Key_Insert &&  bCtrl )
@@ -941,8 +1035,7 @@ bool KDiff3App::eventFilter( QObject* o, QEvent* e )
             init();
          }
 #else
-         KURL::List urlList;
-         KURLDrag::decode( pDropEvent, urlList );
+         KUrl::List urlList = KUrl::List::fromMimeData( pDropEvent->mimeData() );
          if ( canContinue() && !urlList.isEmpty() )
          {
             raise();
@@ -983,7 +1076,9 @@ void KDiff3App::slotFileOpen()
    {
       int result = KMessageBox::warningYesNo(this,
          i18n("You are currently doing a directory merge. Are you sure, you want to abort?"),
-         i18n("Warning"), i18n("Abort"), i18n("Continue Merging") );
+         i18n("Warning"), 
+         KGuiItem( i18n("Abort") ), 
+         KGuiItem( i18n("Continue Merging") ) );
       if ( result!=KMessageBox::Yes )
          return;
    }
@@ -1041,9 +1136,9 @@ void KDiff3App::slotFileOpen()
             m_pDirectoryMergeSplitter->hide();
             init();
 
-            if ( ! m_sd1.isEmpty() && !m_sd1.hasData()  ||
-                 ! m_sd2.isEmpty() && !m_sd2.hasData()  ||
-                 ! m_sd3.isEmpty() && !m_sd3.hasData() )
+            if ( (! m_sd1.isEmpty() && !m_sd1.hasData())  ||
+                 (! m_sd2.isEmpty() && !m_sd2.hasData())  ||
+                 (! m_sd3.isEmpty() && !m_sd3.hasData()) )
             {
                QString text( i18n("Opening of these files failed:") );
                text += "\n\n";
@@ -1112,9 +1207,9 @@ void KDiff3App::slotFileOpen2(QString fn1, QString fn2, QString fn3, QString ofn
       if ( pTotalDiffStatus!=0 )
          return;
 
-      if ( ! m_sd1.isEmpty() && ! m_sd1.hasData()  ||
-           ! m_sd2.isEmpty() && ! m_sd2.hasData()  ||
-           ! m_sd3.isEmpty() && ! m_sd3.hasData() )
+      if ( (! m_sd1.isEmpty() && ! m_sd1.hasData())  ||
+           (! m_sd2.isEmpty() && ! m_sd2.hasData())  ||
+           (! m_sd3.isEmpty() && ! m_sd3.hasData()) )
       {
          QString text( i18n("Opening of these files failed:") );
          text += "\n\n";
@@ -1392,14 +1487,14 @@ void KDiff3App::slotJoinDiffs()
 void KDiff3App::slotConfigure()
 {
    m_pOptionDialog->setState();
-   m_pOptionDialog->incInitialSize ( QSize(0,40) );
+   m_pOptionDialog->incrementInitialSize ( QSize(0,40) );
    m_pOptionDialog->exec();
    slotRefresh();
 }
 
 void KDiff3App::slotConfigureKeys()
 {
-    KKeyDialog::configure(actionCollection(), this);
+    KShortcutsDialog::configure(actionCollection(), KShortcutsEditor::LetterShortcutsAllowed, this);
 }
 
 void KDiff3App::slotRefresh()
@@ -1576,7 +1671,8 @@ void KDiff3App::recalcWordWrap(int nofVisibleColumns) // nofVisibleColumns is >=
    if (bPrinting)
       return;
 
-   m_pOverview->slotRedraw();
+   if (m_pOverview)
+      m_pOverview->slotRedraw();
    if ( m_pDiffTextWindow1 )
    {
       m_pDiffTextWindow1->setFirstLine( m_pDiffTextWindow1->convertDiff3LineIdxToLine( firstD3LIdx ) );
@@ -1593,17 +1689,13 @@ void KDiff3App::recalcWordWrap(int nofVisibleColumns) // nofVisibleColumns is >=
       m_pDiffTextWindow3->update();
    }
 
-   m_pDiffVScrollBar->setRange(0, max2(0, m_neededLines+1 - m_DTWHeight) );
+   if ( m_pDiffVScrollBar )
+      m_pDiffVScrollBar->setRange(0, max2(0, m_neededLines+1 - m_DTWHeight) );
    if ( m_pDiffTextWindow1 )
    {
       m_pDiffVScrollBar->setValue( m_pDiffTextWindow1->convertDiff3LineIdxToLine( firstD3LIdx ) );
 
-      m_maxWidth = max3( m_pDiffTextWindow1->getNofColumns(), 
-                         m_pDiffTextWindow2->getNofColumns(),
-                         m_pDiffTextWindow3->getNofColumns() ) + (m_pOptionDialog->m_bWordWrap ? 0 : 5);
-
-      m_pHScrollBar->setRange(0, max2( 0, m_maxWidth - m_pDiffTextWindow1->getNofVisibleColumns() ) );
-      m_pHScrollBar->setPageStep( m_pDiffTextWindow1->getNofVisibleColumns() );
+      setHScrollBarRange();
       m_pHScrollBar->setValue(0);
    }
 }
@@ -1662,7 +1754,7 @@ bool KDiff3App::improveFilenames( bool bCreateNewInstance )
       {
          f4.addPath( f1.fileName() );
          if ( f4.isFile() && f4.exists() )
-            m_outputFilename = f4.absFilePath();
+            m_outputFilename = f4.absoluteFilePath();
       }
    }
    else if ( f1.isDir() )
@@ -1670,7 +1762,7 @@ bool KDiff3App::improveFilenames( bool bCreateNewInstance )
       m_bDirCompare = true;
       if (bCreateNewInstance)
       {
-         emit createNewInstance( f1.absFilePath(), f2.absFilePath(), f3.absFilePath() );
+         emit createNewInstance( f1.absoluteFilePath(), f2.absoluteFilePath(), f3.absoluteFilePath() );
       }
       else
       {
@@ -1678,6 +1770,7 @@ bool KDiff3App::improveFilenames( bool bCreateNewInstance )
          if (!m_bDefaultFilename) destDir = f4;
          m_pDirectoryMergeSplitter->show();
          if (m_pMainWidget!=0) m_pMainWidget->hide();
+         setUpdatesEnabled(true);
 
          bool bSuccess = m_pDirectoryMergeWindow->init(
             f1, f2, f3,
@@ -1690,11 +1783,11 @@ bool KDiff3App::improveFilenames( bool bCreateNewInstance )
          if (bSuccess)
          {
             m_sd1.reset();
-            if (m_pDiffTextWindow1!=0) m_pDiffTextWindow1->init(0,0,0,0,0,false);
+            if (m_pDiffTextWindow1!=0) m_pDiffTextWindow1->init(0,0,eLineEndStyleDos,0,0,0,0,false);
             m_sd2.reset();
-            if (m_pDiffTextWindow2!=0) m_pDiffTextWindow2->init(0,0,0,0,0,false);
+            if (m_pDiffTextWindow2!=0) m_pDiffTextWindow2->init(0,0,eLineEndStyleDos,0,0,0,0,false);
             m_sd3.reset();
-            if (m_pDiffTextWindow3!=0) m_pDiffTextWindow3->init(0,0,0,0,0,false);
+            if (m_pDiffTextWindow3!=0) m_pDiffTextWindow3->init(0,0,eLineEndStyleDos,0,0,0,0,false);
          }
          slotUpdateAvailabilities();
          return bSuccess;
@@ -1717,7 +1810,9 @@ bool KDiff3App::canContinue()
    {
       int result = KMessageBox::warningYesNoCancel(this,
          i18n("The merge result hasn't been saved."),
-         i18n("Warning"), i18n("Save && Continue"), i18n("Continue Without Saving") );
+         i18n("Warning"), 
+         KGuiItem( i18n("Save && Continue") ), 
+         KGuiItem( i18n("Continue Without Saving") ) );
       if ( result==KMessageBox::Cancel )
          return false;
       else if ( result==KMessageBox::Yes )
@@ -1745,17 +1840,16 @@ void KDiff3App::slotDirShowBoth()
 {
    if( dirShowBoth->isChecked() )
    {
-      if ( m_bDirCompare )
-         m_pDirectoryMergeSplitter->show();
-      else
-         m_pDirectoryMergeSplitter->hide();
+      if ( m_pDirectoryMergeSplitter )
+         m_pDirectoryMergeSplitter->setVisible( m_bDirCompare );
 
       if ( m_pMainWidget!=0 )
          m_pMainWidget->show();
    }
    else
    {
-      if ( m_pMainWidget!=0 )
+      bool bTextDataAvailable = ( m_sd1.hasData() || m_sd2.hasData() || m_sd3.hasData() );
+      if ( m_pMainWidget!=0 &&  bTextDataAvailable )
       {
          m_pMainWidget->show();
          m_pDirectoryMergeSplitter->hide();
@@ -2034,29 +2128,36 @@ void KDiff3App::slotWinToggleSplitterOrientation()
 
 void KDiff3App::slotOverviewNormal()
 {
-   m_pOverview->setOverviewMode( Overview::eOMNormal );
-   m_pMergeResultWindow->setOverviewMode( Overview::eOMNormal );
+   if ( m_pOverview != 0 )
+      m_pOverview->setOverviewMode( Overview::eOMNormal );
+   if ( m_pMergeResultWindow !=0 )
+      m_pMergeResultWindow->setOverviewMode( Overview::eOMNormal );
    slotUpdateAvailabilities();
 }
 
 void KDiff3App::slotOverviewAB()
 {
-   m_pOverview->setOverviewMode( Overview::eOMAvsB );
+   if ( m_pOverview != 0 )
+      m_pOverview->setOverviewMode( Overview::eOMAvsB );
    m_pMergeResultWindow->setOverviewMode( Overview::eOMAvsB );
    slotUpdateAvailabilities();
 }
 
 void KDiff3App::slotOverviewAC()
 {
-   m_pOverview->setOverviewMode( Overview::eOMAvsC );
-   m_pMergeResultWindow->setOverviewMode( Overview::eOMAvsC );
+   if ( m_pOverview != 0 )
+      m_pOverview->setOverviewMode( Overview::eOMAvsC );
+   if ( m_pMergeResultWindow !=0 )
+      m_pMergeResultWindow->setOverviewMode( Overview::eOMAvsC );
    slotUpdateAvailabilities();
 }
 
 void KDiff3App::slotOverviewBC()
 {
-   m_pOverview->setOverviewMode( Overview::eOMBvsC );
-   m_pMergeResultWindow->setOverviewMode( Overview::eOMBvsC );
+   if ( m_pOverview != 0 )
+      m_pOverview->setOverviewMode( Overview::eOMBvsC );
+   if ( m_pMergeResultWindow !=0 )
+      m_pMergeResultWindow->setOverviewMode( Overview::eOMBvsC );
    slotUpdateAvailabilities();
 }
 
@@ -2095,7 +2196,7 @@ static void insertManualDiffHelp( ManualDiffHelpList* pManualDiffHelpList, int w
       int& l2 = i->lastLine( winIdx );
       if (l1>=0 && l2>=0)
       {
-         if ( firstLine<=l1 && lastLine>=l1  ||  firstLine <=l2 && lastLine>=l2 )
+         if ( (firstLine<=l1 && lastLine>=l1)  ||  (firstLine <=l2 && lastLine>=l2) )
          {
             // overlap
             l1 = -1;
@@ -2177,14 +2278,15 @@ void KDiff3App::slotClearManualDiffHelpList()
 
 void KDiff3App::slotUpdateAvailabilities()
 {
+   if (m_pMainSplitter==0)
+      return;
+
    bool bTextDataAvailable = ( m_sd1.hasData() || m_sd2.hasData() || m_sd3.hasData() );
 
    if( dirShowBoth->isChecked() )
    {
-      if ( m_bDirCompare )
-         m_pDirectoryMergeSplitter->show();
-      else
-         m_pDirectoryMergeSplitter->hide();
+      if ( m_pDirectoryMergeSplitter!=0 )
+         m_pDirectoryMergeSplitter->setVisible( m_bDirCompare );
 
       if ( m_pMainWidget!=0 && !m_pMainWidget->isVisible() &&
            bTextDataAvailable && !m_pDirectoryMergeWindow->isScanning()
@@ -2201,8 +2303,8 @@ void KDiff3App::slotUpdateAvailabilities()
    dirShowBoth->setEnabled( m_bDirCompare );
    dirViewToggle->setEnabled(
       m_bDirCompare &&
-      (!m_pDirectoryMergeSplitter->isVisible()  &&  m_pMainWidget!=0 && m_pMainWidget->isVisible() ||
-        m_pDirectoryMergeSplitter->isVisible()  &&  m_pMainWidget!=0 && !m_pMainWidget->isVisible() && bTextDataAvailable )
+      ((!m_pDirectoryMergeSplitter->isVisible()  &&  m_pMainWidget!=0 && m_pMainWidget->isVisible()) ||
+        (m_pDirectoryMergeSplitter->isVisible()  &&  m_pMainWidget!=0 && !m_pMainWidget->isVisible() && bTextDataAvailable) )
       );
 
    bool bDirWindowHasFocus = m_pDirectoryMergeSplitter->isVisible() && m_pDirectoryMergeWindow->hasFocus();
